"use strict";

/* exported logger */

var AbstractGuiElement = require('../AbstractGuiElement');
var DualListbox = require('dual-listbox').DualListbox;
var GuiConnector = require("../../GuiConnector");
var UserPreferences = require("../../map/data/UserPreferences");

var Functions = require('../../functions');
var logger = require('../../logger');

var Promise = require("bluebird");

function ChooseValidatorsDialog(params) {
  AbstractGuiElement.call(this, params);
  var self = this;
  self.createGui();
}

ChooseValidatorsDialog.prototype = Object.create(AbstractGuiElement.prototype);
ChooseValidatorsDialog.prototype.constructor = ChooseValidatorsDialog;

ChooseValidatorsDialog.prototype.createGui = function () {
  var self = this;
  var content = Functions.createElement({
    type: "div",
    style: "display:table"
  });
  content.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<div name='elementTree'/>"
  }));

  content.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<div name='annotatorListBox'/>"
  }));

  self.getElement().appendChild(content);
};

ChooseValidatorsDialog.prototype.setElementType = function (elementType) {
  var self = this;

  var configuration;

  return ServerConnector.getConfiguration().then(function (result) {
    configuration = result;
    return ServerConnector.getLoggedUser();
  }).then(function (user) {
    var element = $("[name='annotatorListBox']", self.getElement())[0];
    Functions.removeChildren(element);

    var validAnnotationSelect = Functions.createElement({
      type: "select",
      className: "minerva-multi-select"
    });

    var miriamTypes = configuration.getMiriamTypes();

    var validAnnotations = user.getPreferences().getElementValidAnnotations(elementType.className);

    for (var i = 0; i < miriamTypes.length; i++) {
      var miriamType = miriamTypes[i];
      var selected = false;
      for (var j = 0; j < validAnnotations.length; j++) {
        if (miriamType.getName() === validAnnotations[j]) {
          selected = true;
        }
      }
      var option = new Option();
      option.value = miriamType.getName();
      option.attributes.selected = selected;
      option.innerHTML = "<div>" + miriamType.getCommonName() + "</div>";
      validAnnotationSelect.appendChild(option);
    }

    element.appendChild(validAnnotationSelect);
    new DualListbox(validAnnotationSelect, {
      addEvent: function (value) {
        var miriamTypes = configuration.getMiriamTypes();
        var miriamType;
        for (var i = 0; i < miriamTypes.length; i++) {
          if (value === miriamTypes[i].getName()) {
            miriamType = miriamTypes[i];
          }
        }
        validAnnotations.push(miriamType.getName());

        var data = new UserPreferences();

        var elementAnnotators = {};
        elementAnnotators[elementType.className] = validAnnotations;
        data.setElementValidAnnotations(elementAnnotators);
        return ServerConnector.updateUserPreferences({user: user, preferences: data}).then(null, GuiConnector.alert);
      },
      removeEvent: function (value) {
        var miriamTypes = configuration.getMiriamTypes();
        var miriamType;
        for (var i = 0; i < miriamTypes.length; i++) {
          if (value === miriamTypes[i].getName()) {
            miriamType = miriamTypes[i];
          }
        }
        var index = validAnnotations.indexOf(miriamType.getName());
        if (index > -1) {
          validAnnotations.splice(index, 1);
        }

        var data = new UserPreferences();

        var elementAnnotators = {};
        elementAnnotators[elementType.className] = validAnnotations;
        data.setElementValidAnnotations(elementAnnotators);
        return ServerConnector.updateUserPreferences({user: user, preferences: data}).then(null, GuiConnector.alert);
      },
      availableTitle: 'Available',
      selectedTitle: 'Used',
      addButtonText: '>',
      removeButtonText: '<',
      addAllButtonText: '>>',
      removeAllButtonText: '<<'
    });
  });

};

ChooseValidatorsDialog.prototype.init = function () {
  var self = this;
  return ServerConnector.getConfiguration().then(function (configuration) {

    var treeData = self.createElementTree(configuration);

    var element = $('[name="elementTree"]', self.getElement());
    element.jstree({
      core: {
        data: treeData
      }
    }).on('ready.jstree', function () {
      element.jstree("open_all");
    }).on("select_node.jstree",
      function (evt, data) {
        return self.setElementType(data.node.data);
      }
    );
  });
};

ChooseValidatorsDialog.prototype.createElementTree = function (configuration) {

  var elementTypes = configuration.getElementTypes();
  var reactionTypes = configuration.getReactionTypes();

  var treeNodes = {
    "lcsb.mapviewer.model.map.BioEntity": {
      text: "BioEntity",
      children: []
    }
  };

  var i;
  for (i = 0; i < elementTypes.length; i++) {
    var type = elementTypes[i];
    var name = type.className;
    if (name.indexOf(".") > 0) {
      name = name.substr(name.lastIndexOf(".") + 1);
    }
    treeNodes[type.className] = {
      text: name,
      data: type,
      children: []
    };
  }

  for (i = 0; i < reactionTypes.length; i++) {
    var type = reactionTypes[i];
    treeNodes[type.className] = {
      text: type.name,
      data: type,
      children: []
    };
  }

  for (var treeNodeName in treeNodes) {
    if (treeNodes.hasOwnProperty(treeNodeName)) {
      var treeNode = treeNodes[treeNodeName];
      if (treeNode.data !== undefined) {
        var parentNode = treeNodes[treeNode.data.parentClass];
        parentNode.children.push(treeNode);
      }
    }
  }

  return treeNodes["lcsb.mapviewer.model.map.BioEntity"];
};

ChooseValidatorsDialog.prototype.destroy = function () {
  $(this.getElement()).dialog("destroy");
};

ChooseValidatorsDialog.prototype.open = function () {
  var self = this;
  var div = self.getElement();
  if (!$(div).hasClass("ui-dialog-content")) {
    $(div).dialog({
      title: "Select valid annotations",
      modal: true,
      width: window.innerWidth / 2,
      height: window.innerHeight / 2

    });
  }

  $(div).dialog("open");
};

module.exports = ChooseValidatorsDialog;
