"use strict";

var GuiConnector = require('../../GuiConnector');

var logger = require('../../logger');
var Promise = require("bluebird");

var ObjectWithListeners = require('../../ObjectWithListeners');

/**
 * Class representing Marker (called sometimes bubble) that we visualize on the
 * map.
 */
function AbstractMarker(params) {
  var self = this;
  // call super constructor
  ObjectWithListeners.call(this);

  self.setIdentifiedElement(params.element);
  self.setIcon(params.element.getIcon());
  self.setCustomMap(params.map);

  self.registerListenerType("onClick");
  if (params.onClick !== undefined) {
    if (typeof params.onClick !== "function") {
      for (var i = 0; i < params.onClick.length; i++) {
        self.addListener("onClick", params.onClick[i]);
      }
    } else {
      self.addListener("onClick", params.onClick);
    }
  }

}

AbstractMarker.prototype = Object.create(ObjectWithListeners.prototype);
AbstractMarker.prototype.constructor = AbstractMarker;

/**
 * Returns identifier of this object.
 *
 * @returns identifier of this object
 */
AbstractMarker.prototype.getId = function () {
  return this.getIdentifiedElement().getId();
};

/**
 * Returns icon of the marker.
 *
 * @returns icon of the marker
 */
AbstractMarker.prototype.getIcon = function () {
  return this._icon;
};

AbstractMarker.prototype.setIcon = function (icon) {
  var self = this;
  self._icon = icon;
  var googleMarker = self.getGoogleMarker();
  if (googleMarker !== undefined) {
    googleMarker.setIcon(GuiConnector.getImgPrefix() + self.getIcon());
  }
};

/**
 * Shows marker on the map.
 */
AbstractMarker.prototype.show = function () {
  if (this.getGoogleMarker() === undefined) {
    logger.warn("Cannot show marker. Marker not initialized");
    return;
  }
  if (this.getGoogleMarker().getMap() !== undefined && this.getGoogleMarker().getMap() !== null) {
    logger.warn("Marker is already shown");
  } else {
    this.getGoogleMarker().setMap(this.getCustomMap().getGoogleMap());
    return;
  }
};
AbstractMarker.prototype.hide = function () {
  if (this.getGoogleMarker() === undefined) {
    logger.warn("Cannot hide marker. Marker not initialized");
    return;
  }
  if (this.getGoogleMarker().getMap() === null || this.getGoogleMarker().getMap() === undefined) {
    logger.warn("Marker is already invisible");
  } else {
    this.getGoogleMarker().setMap(null);
    return;
  }
};

/**
 * Returns {@link AbstractCustomMap} where marker is located.
 *
 * @returns {@link AbstractCustomMap} where marker is located
 */
AbstractMarker.prototype.getCustomMap = function () {
  return this._map;
};

AbstractMarker.prototype.setCustomMap = function (map) {
  this._map = map;
};

/**
 * Returns {@link google.maps.Bounds bounds} of the marker (it's a single
 * point).
 *
 * @returns {@link google.maps.Bounds bounds} of the marker (it's a single
 *          point)
 */
AbstractMarker.prototype.getBounds = function () {
  var marker = this.getGoogleMarker();
  if (marker === undefined || marker === null) {
    logger.warn("Marker not initialized");
    return;
  }
  var bounds = new google.maps.LatLngBounds();
  bounds.extend(marker.position);

  return bounds;
};

/**
 * Set map where the marker should be visualized.
 *
 * @param map
 *          {@link google.maps.Map} where the marker should be visualized.
 */
AbstractMarker.prototype.setMap = function (map) {
  if (this.getGoogleMarker() !== undefined) {
    this.getGoogleMarker().setMap(map);
  } else {
    logger.warn("[" + this.constructor.name + "] Cannot setup map. Marker wasn't initialized. Marker id: "
      + this.getId());
  }
};

/**
 * Initializes google.maps.Marker object connected to this object.
 */
AbstractMarker.prototype._init = function () {
  var self = this;
  var point = this.getCoordinates();
  var latLng = this.getCustomMap().fromPointToLatLng(point);

  this._marker = new google.maps.Marker({
    position: latLng,
    icon: GuiConnector.getImgPrefix() + self.getIcon(),
    optimized: self.getCustomMap().isMarkerOptimization(),
    id: self.getId()
  });

  var onclick = function () {
    return self.onClickHandler().then(null, GuiConnector.alert);
  };
  google.maps.event.addListener(this.getGoogleMarker(), 'click', onclick);
  return Promise.resolve();
};

AbstractMarker.prototype.onClickHandler = function () {
  return this.callListeners("onClick");
};

/**
 * Returns {google.maps.Marker} connected to this object.
 *
 * @returns {google.maps.Marker} connected to this object
 */
AbstractMarker.prototype.getGoogleMarker = function () {
  return this._marker;
};

/**
 * Abstract function returning string marker type.
 *
 * @returns string marker type
 */
AbstractMarker.prototype.getType = function () {
  return this.getIdentifiedElement().getType();
};

AbstractMarker.prototype.getModelId = function () {
  return this.getIdentifiedElement().getModelId();
};

AbstractMarker.prototype.getIdentifiedElement = function () {
  return this._identifiedElement;
};

AbstractMarker.prototype.updateIdentifiedElement = function (identifiedElement) {
  this.setIcon(identifiedElement.getIcon());
};

AbstractMarker.prototype.setIdentifiedElement = function (identifiedElement) {
  this._identifiedElement = identifiedElement;
};

module.exports = AbstractMarker;
